package net.sf.doolin.gui.template.field;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JScrollPane;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.field.AbstractPropertyFieldDescriptor;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.support.FieldActionContext;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.template.component.ActionCallback;
import net.sf.doolin.gui.template.component.JTemplate;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.template.Template;
import net.sf.doolin.template.TemplateEngine;
import net.sf.doolin.template.TemplateManager;
import net.sf.doolin.util.factory.DataFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Required;

/**
 * Read-only field that displays a template.
 * 
 * @param <V>
 *            Type for the view
 * 
 * @author Damien Coraboeuf
 */
public class FieldTemplate<V> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private final static Logger log = LoggerFactory.getLogger(FieldTemplate.class);

	private TemplateManager templateManager;

	private String templatePath;

	private String templateType = TemplateEngine.VELOCITY;

	private String contentFormat;

	private String propertyName = "model";

	private Map<String, DataFactory<?>> context;

	private Map<String, GUIAction> actionMap = new HashMap<String, GUIAction>();

	@Override
	public Field<V> createField(final GUIView<V> view) {
		// Creates the template component
		final JTemplate jTemplate = new JTemplate();
		// Field
		final SimpleField<V> field = new SimpleField<V>(view, this,
				new JScrollPane(jTemplate));
		// Sets the template manager
		jTemplate.setTemplateManager(this.templateManager);
		// Sets the action call-back
		jTemplate.setActionCallback(new ActionCallback() {

			@Override
			public void doAction(String actionId, Map<String, String> params) {
				// Gets the action
				GUIAction action = FieldTemplate.this.actionMap.get(actionId);
				if (action != null) {
					// Action context
					FieldActionContext<V> context = new FieldActionContext<V>(
							field, params);
					// Executes the action
					action.execute(context);
				}
				// No action
				else {
					log.error(String.format(
							"No action is defined with ID = %s for field %s",
							actionId, getName()));
				}
			}

		});
		// Sets the template path
		jTemplate.setPath(this.templatePath);
		// Sets the template type
		jTemplate.setType(this.templateType);
		// Sets the content format
		jTemplate.setFormat(this.contentFormat);
		// Initial context
		Object model = getProperty(view);
		Map<String, Object> computedContext = computeContext(model);
		// Sets the context
		jTemplate.setContext(computedContext);
		// Listens to the model
		subscribe(view, new Runnable() {

			@Override
			public void run() {
				Object model = getProperty(view);
				Map<String, Object> computedContext = computeContext(model);
				jTemplate.setContext(computedContext);
			}
		});
		// OK
		return field;
	}

	/**
	 * Gets the map of actions in the template. Each logical key in the template
	 * content can be mapped to a {@link GUIAction} to execute.
	 * 
	 * @return Map of actions
	 */
	public Map<String, GUIAction> getActionMap() {
		return this.actionMap;
	}

	/**
	 * Gets the content format for this template. It may be one of the
	 * following:
	 * <ul>
	 * <li>text/plain</li>
	 * <li>text/html</li>
	 * </ul>
	 * 
	 * @return MIME type that defines the content
	 */
	@Required
	public String getContentFormat() {
		return this.contentFormat;
	}

	/**
	 * Gets the context to define for this template. When the template context
	 * is created, each {@link DataFactory} is called in order to get an actual
	 * object to put in the template context with the associated key.
	 * 
	 * @return Map of {@link DataFactory} instances
	 * @see Template#generate(java.io.Writer, Map)
	 */
	public Map<String, DataFactory<?>> getContext() {
		return this.context;
	}

	/**
	 * Gets the name which is used to identify the field property in the
	 * template.
	 * 
	 * @return Name of the model
	 */
	public String getPropertyName() {
		return this.propertyName;
	}

	/**
	 * Gets the template manager which is used by the template
	 * 
	 * @return Template manager
	 */
	public TemplateManager getTemplateManager() {
		return this.templateManager;
	}

	/**
	 * Gets the path to the template.
	 * 
	 * @return Path
	 * @see TemplateEngine#getTemplate(String)
	 */
	public String getTemplatePath() {
		return this.templatePath;
	}

	/**
	 * Gets the template type used to get the {@link TemplateEngine}
	 * 
	 * @return Template type
	 * @see TemplateManager#getTemplateEngine(String)
	 * @see TemplateManager#getTemplateEngineNames()
	 */
	public String getTemplateType() {
		return this.templateType;
	}

	/**
	 * Sets the map of actions in the template. Each logical key in the template
	 * content can be mapped to a {@link GUIAction} to execute. Actions are
	 * executed only for {@linkplain #setContentFormat(String) HTML} content,
	 * when a "action:&lt;actionKey&gt;" hyperlink is defined.
	 * 
	 * @param actionMap
	 *            Map of actions
	 */
	public void setActionMap(Map<String, GUIAction> actionMap) {
		this.actionMap = actionMap;
	}

	/**
	 * Sets the content format for this template. It may be one of the
	 * following:
	 * <ul>
	 * <li>text/plain</li>
	 * <li>text/html</li>
	 * </ul>
	 * 
	 * @param contentFormat
	 *            MIME type that defines the content
	 */
	public void setContentFormat(String contentFormat) {
		this.contentFormat = contentFormat;
	}

	/**
	 * Sets the context to define for this template. When the template context
	 * is created, each {@link DataFactory} is called in order to get an actual
	 * object to put in the template context with the associated key.
	 * 
	 * @param context
	 *            Map of {@link DataFactory} instances
	 * @see Template#generate(java.io.Writer, Map)
	 */
	public void setContext(Map<String, DataFactory<?>> context) {
		this.context = context;
	}

	/**
	 * Sets the name which is used to identify the field property in the
	 * template. It is set by default to "model".
	 * 
	 * @param propertyName
	 *            Name of the model
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	/**
	 * Sets the template manager which is used by the template
	 * 
	 * @param templateManager
	 *            Template manager to use
	 */
	@Autowired(required = true)
	public void setTemplateManager(TemplateManager templateManager) {
		this.templateManager = templateManager;
	}

	/**
	 * Sets the path to the template.
	 * 
	 * @param templatePath
	 *            Path
	 * @see TemplateEngine#getTemplate(String)
	 */
	@Required
	public void setTemplatePath(String templatePath) {
		this.templatePath = templatePath;
	}

	/**
	 * Sets the template type used to get the {@link TemplateEngine}
	 * 
	 * @see TemplateManager#getTemplateEngine(String)
	 * @see TemplateManager#getTemplateEngineNames()
	 * @param templateType
	 *            Template type
	 */
	public void setTemplateType(String templateType) {
		this.templateType = templateType;
	}

	private Map<String, Object> computeContext(Object model) {
		Map<String, Object> computedContext = new HashMap<String, Object>();
		// Property
		computedContext.put(this.propertyName, model);
		// Additional context entries
		if (this.context != null) {
			for (Map.Entry<String, DataFactory<?>> entry : this.context
					.entrySet()) {
				String entryName = entry.getKey();
				DataFactory<?> dataFactory = entry.getValue();
				Object item = dataFactory.create(model);
				computedContext.put(entryName, item);
			}
		}
		return computedContext;
	}

}
